@@ -713,7 +713,7 @@ function _validateStdio(stdio, sync) {
     } else if (typeof stdio === 'number' || typeof stdio.fd === 'number') {
       acc.push({
         type: 'fd',
-        fd: stdio.fd || stdio
+        fd: typeof stdio === 'number' ? stdio : stdio.fd
       });
     } else if (getHandleWrapType(stdio) || getHandleWrapType(stdio.handle) ||
                getHandleWrapType(stdio._handle)) {
