@@ -436,15 +436,6 @@ exports.hasMultiLocalhost = function hasMultiLocalhost() {
   return ret === 0;
 };
 
-exports.isValidHostname = function(str) {
-  // See http://stackoverflow.com/a/3824105
-  var re = new RegExp(
-    '^([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\\-]{0,61}[a-zA-Z0-9])' +
-    '(\\.([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\\-]{0,61}[a-zA-Z0-9]))*$');
-
-  return !!str.match(re) && str.length <= 255;
-};
-
 exports.fileExists = function(pathname) {
   try {
     fs.accessSync(pathname);
