@@ -1,15 +1,17 @@
+'use strict';
 var common = require('../common');
 var assert = require('assert');
 var fs = require('fs');
 var spawn = require('child_process').spawn;
 
 // Fails with EINVAL on SmartOS, EBUSY on Windows.
-if (process.platform === 'sunos' || process.platform === 'win32') {
+if (process.platform === 'sunos' || common.isWindows) {
   console.log('1..0 # Skipped: cannot rmdir current working directory');
   return;
 }
 
 var dirname = common.tmpDir + '/cwd-does-not-exist-' + process.pid;
+common.refreshTmpDir();
 fs.mkdirSync(dirname);
 process.chdir(dirname);
 fs.rmdirSync(dirname);
