@@ -1,9 +1,10 @@
+'use strict';
 var common = require('../common');
 var assert = require('assert');
 
 if (!common.hasCrypto) {
   console.log('1..0 # Skipped: missing crypto');
-  process.exit();
+  return;
 }
 var tls = require('tls');
 
